<?php
/*
YaotGui Controls


This file is part of YaotGui.

YaotGui is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

YaotGui is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with YaotGui.  If not, see <http://www.gnu.org/licenses/>.

Copyright 2008 Thomas Axelsson

*/

/// Include default styles for the controls
/** Put this before you customize the controls with your own style tags!
 */
function defaultStyles()
{
   echo '<link rel="stylesheet" type="text/css" href="lib.css">';
}


  /// The class which all lib objects are derived from
  /** It has a tag attribute to store arbitrary user data */
abstract class LibObject
{
   /// Stores the user assigned tag
   protected $_tag;

   //accessors
   /// Set the user assigned tag
   /** \param $value an arbitrary value
       \sa getTag
   */
   public final function setTag($value)
   {
      $this->_tag = $value;
   }

   /// Get the user assigned tag
   /** \return an arbitrary value
       \sa setTag
    */
   public final function getTag()
   {
      return $this->_tag;
   }
}

/// The class which all controls derives from
/** Stores basic control information */
abstract class LibControl extends LibObject
{
   /// The control name - as a string
   protected $_type;

   /// The control id
   /** Appears in html code as <... id="<i>myid</i>" ...> */
   protected $_id;

   /// Prefix to put on varibles and function in javascript code
   /** It's value is _<i>class-id</i>__ */
   protected $_jsPrefix;
   
   /// Access to the control element in javascript
   /** Example:
       $js = "
       function alertId()
       {
           alert($this->_element.id);
       }";
    */
   protected $_element;
   /// The CSS class
   /** Used to customize the control look,
       ie &lt;style&gt;my_control_class {font-weight: bold;}&lt;/style&gt;
    */
   protected $_class;

   // The default values CSS class
//   protected $_defaultClass;

   /// Prints the HTML for the current control
   /** Also any additional javascript functions needed.
       They will be called __control-id_function-name(...)
    */
   public abstract function printHtml();

   /// Creates a new control
   /** This constructor is never called directly
       \param $id The HTML/Javascript DOM id
       \param $class The CSS class
    */
   function __construct($id, $class)
   {
      $this->_id = $id;
      $this->_element = "document.getElementById('$id')";
      $this->_class = $class;
      $this->_jsPrefix = '__' . $id . '_';
      $this->_type = get_class($this);
//      $this->_defaultClass = '__' . $this->_type . '_default';
   }

   //helpers
   /// Prints a new DIV start tag, with the control id
   /** Will appear as &lt;div id="control-id" <i>$extraAttr</i>&gt;
      
       \param $extraAttr Any extra attributes to the DIV
    */
   protected function printIdDivStart($extraAttr='')
   {
      $this->printDivStart("id=\"$this->_id\" $extraAttr");
   }

   /// Prints a new DIV start tag, with the control id and class
   /** Will appear as &lt;div id="control-id" class="control-class" <i>$extraAttr</i>&gt;
      
       \param $extraAttr Any extra attributes to the DIV
    */
   protected function printIdClassDivStart($extraAttr='')
   {
      $this->printIdDivStart("class=\"" . $this->getDefaultClass() . " $this->_class\" $extraAttr");
   }

   /// Prints a new DIV start tag
   /** Will appear as &lt;div <i>$extraAttr</i>&gt;
      
       \param $extraAttr Any extra attributes to the DIV
    */
   protected static function printDivStart($extraAttr='')
   {
      echo "<div $extraAttr>";
   }

   /// Prints a new DIV end tag
   /** &lt;/div&gt; */
   protected static function printDivEnd()
   {
      echo '</div>';
   }

   protected static function printScriptStart()
   {
      echo '<script type="text/javascript">';
   }

   protected static function printScriptEnd()
   {
      echo '</script>';
   }

   protected static function printScript($content)
   {
      self::printScriptStart();
      echo $content;
      self::printScriptEnd();
   }

/*   protected static function printHiddenValue($name, $value)
   {
      echo "<input type=\"hidden\" name=\"$name\" value=\"$value\">";
   }

   protected static function printHiddenValues($values)
   {
      foreach($values as $name=>$value)
      {
	 self::printHiddenValue($name, $value);
      }
   }*/

   protected static function printIdTableStart($extraAttr='')
   {
      self::printTableStart("id=\"$this->_id\" $extraAttr");
   }

   protected static function printIdClassTableStart($extraAttr='')
   {
      self::printIdTableStart("class=\"" . $this->getDefaultClass() . " $this->_class\" $extraAttr");
   }

   protected static function printTableStart($extraAttr='')
   {
      echo "<table $extraAttr>";
   }

   protected static function printTableEnd()
   {
      echo '</table>';
   }

   protected static function printRowStart()
   {
      echo '<tr>';
   }

   protected static function printRowEnd()
   {
      echo '</tr>';
   }

   protected function printIdColumnStart($extraAttr='')
   {
      $this->printColumnStart("id=\"$this->_id\" $extraAttr");
   }

   protected function printIdClassColumnStart($extraAttr='')
   {
      $this->printIdColumnStart("class=\"" . $this->_getDefaultClass() . " $this->_class\" $extraAttr");
   }

   protected static function printColumnStart($extraAttr='')
   {
      echo "<td $extraAttr>";
   }

   protected static function printColumnEnd()
   {
      echo '</td>';
   }

   protected static function printColumn($context)
   {
      $this->printColumnStart();
      echo $context;
      $this->printColumnEnd();
   }

   public function getId()
   {
      return $this->_id;
   }

   public function getClass()
   {
      return $this->_class;
   }

   /// Returns the current the class name as a string
   /** This must be overridden because get_class() will return the top parent of any derived class
    */
   public static abstract function getDefaultClass();
/*   {
      return '__' . get_class() . '_default';
      }*/

   public static function getDefaultClassByType($type)
   {
      return "__{$type}_default";
   }

   /// Returns a string with both the default and current class
   public function getClassString()
   {
      return self::getClassStringByNames($this->getDefaultClass(), $this->getClass());
   }

   public static function getClassStringByNames($default, $current, $suffix = NULL)
   {
      if($suffix !== NULL)
	 return "{$default}_$suffix {$current}_$suffix";
      return "$default $current";
   }
}

?>
